#region Copyright (c) 2005
/*
{*******************************************************************}
{                                                                   }
{       Copyright (c) 2005                                          }
{		Author: Liewen Huang                                        }
{       ALL RIGHTS RESERVED                                         }
{                                                                   }
{*******************************************************************}
*/
#endregion
using System;
using System.Collections;
using System.IO;

namespace SampleLRM
{
	/// <summary>
	/// The class for the collection of Excel sheets, that is, the translation sheets.
	/// </summary>
	public class ResourceRepository : Hashtable
	{
		public ResourceRepository()
		{			
		}

		public Hashtable Initialize()
		{
			if (!Directory.Exists(SampleLRM.ResourceRepositoryPath))
				Directory.CreateDirectory(SampleLRM.ResourceRepositoryPath);

			System.Reflection.Missing missing = System.Reflection.Missing.Value;

			Excel.Application oEA = new Excel.Application();
			Excel.Workbooks oWBs;

			Hashtable culturesFoundInXlsFiles = new Hashtable();

			// load all xls files existing in the resource repository folder.
			foreach (FileInfo fi in (new DirectoryInfo(SampleLRM.ResourceRepositoryPath)).GetFiles())
			{
				if (fi.Extension.ToLower() != ".xls")
					continue;

				oWBs = oEA.Workbooks;
				// You may need to modify this method if you are not using Excel 11.0.
				oWBs.Open(fi.FullName, missing, true, missing, missing, missing, missing, missing, missing, missing, missing, missing, missing, missing, missing);

				Excel._Workbook oWB;
				oWB = oWBs[1];

				foreach (Excel.Worksheet oSheet in oWB.Sheets)
					if (!oSheet.Name.StartsWith("Sheet"))
					{
						string culture = oSheet.Name;

						if (TranslationSheet.IsSupportdCultureInfo(System.Globalization.CultureTypes.AllCultures, culture))
							culturesFoundInXlsFiles[culture] = culture;

						Excel.Range oRange;
						oRange = oSheet.UsedRange;

						object[,] saRet;
						saRet = (object[,])oRange.Value2;

						if (saRet == null)
							continue;

						int lb = saRet.GetLowerBound(0);
						int ub = saRet.GetUpperBound(0);

						// build the translation sheet.
						TranslationSheet tSheet = this[culture];
						tSheet.XLSFile = fi.FullName;

						for (int i = lb; i <= ub; i++)
						{
							object value;

							value = saRet[i, 1];
							string invariantString = (value == null) ? null : value.ToString();

							if (invariantString == null)
								continue;

							value = saRet[i, 2];
							string localizedValue = (value == null) ? null : value.ToString();

							value = saRet[i, 3];
							string name = (value == null) ? null : value.ToString();

							value = saRet[i, 4];
							string path = (value == null) ? null : value.ToString();
						
							tSheet.Add(invariantString, path, name, localizedValue);
						}
					}
		
				oWB.Close(missing, missing, missing);
				oWBs.Close();
			}
			oEA.Quit();

			return culturesFoundInXlsFiles;
		}

		public TranslationSheet this[string culture]
		{
			get
			{
				if (!base.ContainsKey(culture))
				{
					TranslationSheet tSheet = new TranslationSheet(culture);
					this[culture] = tSheet;
				}

				return (TranslationSheet)base[culture];
			}
			set
			{
				base[culture] = value;
			}
		}

		public void Save()
		{
			foreach (DictionaryEntry de in this)
				((TranslationSheet)de.Value).Save();		
		}

		//  Use this to determine if it is needed to create new language-specific resource files.
		public DateTime CultureLastModifiedTime(string culture)
		{
			if (base.ContainsKey(culture))
				return ((TranslationSheet)base[culture]).LastModified;
			else
				return DateTime.Now;
		}
	}
}
