#region Copyright (c) 2005
/*
{*******************************************************************}
{                                                                   }
{       Copyright (c) 2005                                          }
{		Author: Liewen Huang                                        }
{       ALL RIGHTS RESERVED                                         }
{                                                                   }
{*******************************************************************}
*/
#endregion
using System;
using System.Xml;
using System.IO;
using System.Collections;
using System.Xml.XPath;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using System.Text;
using System.Diagnostics;
using System.Globalization;

namespace SampleLRM
{
	/// <summary>
	/// The public class of this component.
	/// </summary>
	public class SampleLRM
	{
		// public options.
		public static string ResourceRepositoryPath = string.Empty;
		public static bool WriteUniqueResourceOnly = false;
		public static bool GenerateRes = false; 
		public static bool RebuildAccessKey = false;
		public static event PerformRuleEvaluationHandler PerformRuleEvaluation;

		public const int MaxLengthAnExcelCellCanHold = 911;

		internal static ResourceRepository _resourceRepository;

		public SampleLRM()
		{
		}

		private static XmlDocument ReadXmlDocument(string fileName)
		{
			XmlDocument doc = new XmlDocument();
			doc.PreserveWhitespace = false;

			string file = GetFileContents(fileName);

			try
			{
				doc.LoadXml(file);
			}
			catch
			{
				return null;
			}

			return doc;
		}

		/// <summary>
		/// Generate language-specific resource files for .NET projects.
		/// </summary>
		/// <param name="projNames">the projects that need to add the resource files</param>
		/// <param name="requireToSupportCultures">the targets cultures</param>
		public static void GenerateResourceFileForProjects(string[] projNames, 
			string[] requireToSupportCultures)
		{
			_resourceRepository = new ResourceRepository();

			// reads the excel files
			Hashtable cultures = _resourceRepository.Initialize();

			foreach (string cult in requireToSupportCultures)
				cultures[cult] = cult;

			// generate the language-specific resource files for each project.
			foreach (string projName in projNames)
				GenerateResourceFileForProject(projName, cultures.Values);

			// updates the excel files.
			_resourceRepository.Save();
		}

		private static bool IsFileNameContainsCulture(string fileName)
		{
			if (Path.GetExtension(fileName) != ".resx")
				return false;

			string fileNo = Path.GetFileNameWithoutExtension(fileName);

			int l = fileNo.LastIndexOf(".");

			if (l < 0)
				return false;

			return TranslationSheet.IsSupportdCultureInfo(CultureTypes.AllCultures, fileNo.Substring(l + 1));
		}

		//  generate language-specific resource files for a project for all the target cultures.
		private static void GenerateResourceFileForProject(string projName, ICollection cultures) 
		{
			System.Diagnostics.Debug.WriteLine(projName);

			const string PATH_TAG = "RelPath";

			// only csproj and vbproj files are supported.
			if (!File.Exists(projName) 
				|| (string.Compare(Path.GetExtension(projName), ".csproj", true) == 0 
				&& string.Compare(Path.GetExtension(projName), ".vbproj", true) == 0) 
				|| new FileInfo(projName).Length == 0)
				return;

			// set the file to writable.
			SetFileWritable(projName);

			XmlDocument xmlDoc = ReadXmlDocument(projName);
			
			if (xmlDoc == null)
				return;

			string relPathStr = string.Empty;

			string fileNodesPath;
			if (string.Compare(Path.GetExtension(projName), ".csproj", true) == 0)
				fileNodesPath = "//VisualStudioProject/CSHARP/Files/Include/File";
			else if (string.Compare(Path.GetExtension(projName), ".vbproj", true) == 0)
				fileNodesPath = "//VisualStudioProject/VisualBasic/Files/Include/File";
			else
				return;

			XmlNodeList fileNodes = xmlDoc.SelectNodes(fileNodesPath);

			Hashtable allFileNodes = new Hashtable();

			bool someNodesAdded = false;

			foreach (System.Xml.XmlNode node in fileNodes)
				allFileNodes.Add(node.Attributes.GetNamedItem(PATH_TAG).InnerText, 1);

			XmlNode fileNodesRoot = xmlDoc.SelectNodes(fileNodesPath).Item(0).ParentNode;

			foreach(System.Xml.XmlNode fileNode in fileNodes) 
			{
				if (fileNode.Attributes.GetNamedItem(PATH_TAG) != null) 
				{
					relPathStr = fileNode.Attributes.GetNamedItem(PATH_TAG).InnerText;

					// skip any language-specific resource files.
					if (IsFileNameContainsCulture(relPathStr))
						continue;

					if( relPathStr.ToLower().IndexOf(".resx") >= 0 )  
					{
						if (relPathStr.ToLower().EndsWith(".aspx.resx"))
							continue;

						if (!File.Exists(Path.Combine(Path.GetDirectoryName(projName), relPathStr)))
							continue;

						XmlNode cloned;

						foreach (string cult in cultures)
						{
							// new node for the language-specific resource file.
							cloned = fileNode.Clone();
							string newResourceFileName = relPathStr.Insert(relPathStr.LastIndexOf("."),"." + cult);
							cloned.Attributes.GetNamedItem(PATH_TAG).InnerText = newResourceFileName;

							if (HasGeneratedLanguageSpecificResourceFile(Path.GetDirectoryName(projName), relPathStr, cult, newResourceFileName))
							{
								// only add it if not existing
								if (!allFileNodes.ContainsKey(cloned.Attributes.GetNamedItem(PATH_TAG).InnerText))
								{
									fileNodesRoot.AppendChild(cloned);
									someNodesAdded = true;
								}
							}
						}
					}
				}
			}

			// save the project file.
			if (someNodesAdded)
				xmlDoc.Save(projName);
		}

		private static Hashtable resxCache = new Hashtable();

		// Try to generate the language-specific resource file.
		private static bool HasGeneratedLanguageSpecificResourceFile(string folder, string sourcePath, string culture, string outPath)
		{
			string languageSpecificResourceFilePath = Path.Combine(folder, outPath);
			string textBasedResourceFile = Path.ChangeExtension(languageSpecificResourceFilePath, ".txt");
			string defaultResourceFile = Path.Combine(folder, sourcePath);
			string prevTextString = string.Empty;
			string curTextString = string.Empty;

			if (File.Exists(textBasedResourceFile))
			{
				// return if it doesn't need to re-generate the resource file.
				if (File.GetLastWriteTime(textBasedResourceFile) > File.GetLastWriteTime(defaultResourceFile) &&
					File.GetLastWriteTime(textBasedResourceFile) > _resourceRepository.CultureLastModifiedTime(culture) &&
					File.Exists(languageSpecificResourceFilePath) && 
					File.GetLastWriteTime(languageSpecificResourceFilePath) >= File.GetLastWriteTime(textBasedResourceFile))
					return true;

				prevTextString = GetFileContents(textBasedResourceFile);
			}

			// cache nodes.
			if (!resxCache.ContainsKey(defaultResourceFile))
			{
				XmlDocument doc = ReadXmlDocument(defaultResourceFile);

				resxCache[defaultResourceFile] = new XmlNodeList[]{doc.SelectNodes("/root/data")};
			}

			XmlNodeList dataNodes = ((XmlNodeList[])resxCache[defaultResourceFile])[0];

			foreach(System.Xml.XmlNode node in dataNodes) 
			{
				// not translate if type or mimetype is not null.
				if (node.Attributes.GetNamedItem("type") != null ||
					node.Attributes.GetNamedItem("mimetype") != null)
					continue;

				if (node.FirstChild == null)
					continue;

				string name = node.Attributes.GetNamedItem("name").InnerText;
				string word = node.FirstChild.InnerText;
				
				if (word == string.Empty)
					continue;

				if (NeedsToTranslate(name, word, defaultResourceFile))
				{
					ResourceRow entry = _resourceRepository[culture][word, defaultResourceFile, name];
					if (word != entry.LocalizedValue 
						&& ReplaceAltKey(word) != entry.LocalizedValue
						&& entry.LocalizedValue != string.Empty)
						curTextString += name + "=" + entry.LocalizedValue + "\r\n";
				}
			}


			if (curTextString.Length > 0)
			{
				// only needs to generate a new xml-base resource file if the new text file is different from the
				// old one, as well as file not existing or file is not new enough.
				if (curTextString != prevTextString || 
					!File.Exists(languageSpecificResourceFilePath) || 
					File.GetLastWriteTime(languageSpecificResourceFilePath) < File.GetLastWriteTime(textBasedResourceFile))
				{
					SetFileWritable(textBasedResourceFile);

					StreamWriter sw = null;

					// may need to try a few time to open the file if it belongs to more than one project.
					int i = 0;
					do
					{
						try
						{
							sw = new StreamWriter(textBasedResourceFile);
						}
						catch
						{
							System.Threading.Thread.Sleep(1000);
							i++;
						}
					} while (sw == null && i < 5);

					sw.Write(curTextString);
					sw.Close();

					// use resgen command to generate resource files.
					if (SampleLRM.GenerateRes)
					{
						SetFileWritable(languageSpecificResourceFilePath);

						ProcessStartInfo procStartInfo = new ProcessStartInfo("resgen", "\"" + textBasedResourceFile + "\" \"" + languageSpecificResourceFilePath + "\"");
						procStartInfo.WindowStyle = ProcessWindowStyle.Hidden;
						procStartInfo.UseShellExecute = false;
						procStartInfo.RedirectStandardError = true;

						Process process = new Process();
						process.StartInfo = procStartInfo;
						process.Start();
					}
				}

				return true;
			}
			else
			{
				File.Delete(textBasedResourceFile);
				return false;
			}
		}

		// change a file's attribute to writable.
		internal static void SetFileWritable(string fileName)
		{
			if (File.Exists(fileName))
			{
				if (((File.GetAttributes(fileName) & FileAttributes.ReadOnly) > 0) ) 
					File.SetAttributes(fileName, File.GetAttributes(fileName) ^ FileAttributes.ReadOnly);
			}
		}


		private static bool NeedsToTranslate(string name, string invariantString, string path)
		{
			if (name == null || name == string.Empty)
				return false;

			if (name.StartsWith(">>"))
				return false;

			if (null != PerformRuleEvaluation)
			{
				PerformRuleEvaluationArgs e = new PerformRuleEvaluationArgs();

				e.InvariantString = invariantString;
				e.Name = name;

				// fire the PerformRuleEvalution event and let the EXE handles it.
				foreach (Delegate d in PerformRuleEvaluation.GetInvocationList())
				{
					d.DynamicInvoke(new object [] { null, e });
					if (e.Handled)
					{
						if (!e.NeedToBeTranslated)
						{
							ResourceRow entry = _resourceRepository[e.Cause][invariantString, path, name];
							return false;
						}
						break;
					}
				}
			}

			return true;
		}

		//  reconstruct the string by moving the access key to the end of the string and 
		//  putting parentheses around it.
		internal static string ReplaceAltKey(string s)
		{
			if (!SampleLRM.RebuildAccessKey)
				return s;

			Regex digitregex = new Regex("(?<altkey>&[0-9a-zA-Z])");
 
			MatchCollection ms = digitregex.Matches(s);
			if (ms.Count == 1)
			{
				string found = ms[0].Groups["altkey"].Value;
				return s.Replace("&", "") + " (" + found + ")";
			}
			else
				return s;
		}

		private static string GetFileContents(string fileName)
		{
			StreamReader sr = new StreamReader(fileName);

			string ret = sr.ReadToEnd();

			sr.Close();

			return ret;
		}
	}
	
	public delegate void PerformRuleEvaluationHandler(object sender, PerformRuleEvaluationArgs e);

	public class PerformRuleEvaluationArgs
	{
		private bool _handled = false;
		private string _cause = string.Empty;
		private bool _needToBeTranslated = false;
		private string _invariantString = string.Empty;
		private string _name = string.Empty;

		internal PerformRuleEvaluationArgs() {}

		public bool Handled
		{
			get { return _handled; }
			set { _handled = value; }
		}

		public string Cause 
		{
			get { return _cause; }
			set { _cause = value; }
		}

		public bool NeedToBeTranslated
		{
			get { return _needToBeTranslated; }
			set { _needToBeTranslated = value; }
		}

		public string InvariantString
		{
			get { return _invariantString; }
			set { _invariantString = value; }
		}

		public string Name
		{
			get { return _name; }
			set { _name = value; }
		}
	}
}
