#region Copyright (c) 2005
/*
{*******************************************************************}
{                                                                   }
{       Copyright (c) 2005                                          }
{		Author: Liewen Huang                                        }
{       ALL RIGHTS RESERVED                                         }
{                                                                   }
{*******************************************************************}
*/
#endregion
using System;
using System.Collections;
using System.IO;
using System.Text.RegularExpressions;
using System.Globalization;

namespace SampleLRM
{
	/// <summary>
	/// The class for the collection of translations for a culture.
	/// </summary>
	public class TranslationSheet : Hashtable
	{
		private string _culture;

		public TranslationSheet(string culture)
		{
			_culture = culture;
		}

		public void Add(string invariantString, string path, string name, string localizedValue)
		{
			ResourceRow entry = this[invariantString, path, name];
			entry.LocalizedValue = localizedValue;
			entry.IsANewResource = false;
		}

		public ResourceRow this[string invariantString, string path, string name]
		{
			get
			{
				invariantString = invariantString.Replace("\t", " ");

				if (invariantString.StartsWith("'"))
					invariantString = invariantString.Substring(1);

				// Excel's default format uses "True" and "False".
				if (string.Compare(invariantString, "true", true) == 0)
					invariantString = "True";

				if (string.Compare(invariantString, "false", true) == 0)
					invariantString = "False";

				if (string.Compare(name, "true", true) == 0)
					name = "True";

				if (string.Compare(name, "false", true) == 0)
					name = "False";

				if (invariantString.Length > SampleLRM.MaxLengthAnExcelCellCanHold)
					invariantString = invariantString.Substring(0, SampleLRM.MaxLengthAnExcelCellCanHold);

				if (!this.ContainsKey(invariantString))
					this[invariantString] = new InvariantString(invariantString, path, name, SampleLRM.ReplaceAltKey(invariantString), true);

				return ((InvariantString)this[invariantString])[invariantString, path, name];
			}
		}

		// Save the changes.
		public void Save()
		{
			System.Reflection.Missing missing = System.Reflection.Missing.Value;

			if (IsSupportdCultureInfo(System.Globalization.CultureTypes.SpecificCultures, _culture))
					return;

			// prepare the Excel file.
			Excel.Application oEA = new Excel.Application();
			Excel.Workbooks oWBs;
			oWBs = oEA.Workbooks;

			Excel._Workbook oWB;

			bool xlsExits = File.Exists(XLSFile);
			if (xlsExits)
			{
				SampleLRM.SetFileWritable(XLSFile);

				// You may need to modify this method if you are not using Excel 11.0.
				oWBs.Open(XLSFile, missing, missing, missing, missing, missing, missing, missing, missing, missing, missing, missing, missing, missing, missing);
				oWB = oWBs[1];
			}
			else
				oWB = (Excel._Workbook)(oEA.Workbooks.Add( missing ));

			Excel._Worksheet oSheet = null;

			foreach (Excel.Worksheet sheet in oWB.Sheets)
				if (sheet.Name == _culture)
				{
					oSheet = sheet;
					break;
				}

			bool createNewSheet = false;
			
			// create the sheet if needed.
			if (oSheet == null)
			{
				oSheet = (Excel.Worksheet)oWB.Sheets.Add(missing, missing, missing, missing);
				oSheet.Name = _culture;

				createNewSheet = true;
			}

			ArrayList list = new ArrayList();

			// prepare the data.
			foreach (DictionaryEntry w in this)
			{
				InvariantString wg = (InvariantString)w.Value;

				foreach (DictionaryEntry p in wg)
				{
					ResourcePath pg = (ResourcePath)p.Value;

					bool savedOne = false;

					foreach (DictionaryEntry e in pg)
					{
						ResourceRow entry = (ResourceRow)e.Value;

						if (entry.IsANewResource)
						{
							list.Add(new string[]{entry.InvariantStringValue, entry.LocalizedValue, entry.ResourceName, entry.ResourceFilePath, "Y"});

							savedOne = true;

							if (SampleLRM.WriteUniqueResourceOnly)
								break;
						}
					}

					if (SampleLRM.WriteUniqueResourceOnly && savedOne)
						break;
				}
			}

			if (list.Count > 0)
			{
				// assign the data to the sheet.
				Excel.Range oRange;
				oRange = oSheet.get_Range("A" + (oSheet.UsedRange.Rows.Count + (createNewSheet ? 0 : 1)).ToString(), missing);
				oRange = oRange.get_Resize(list.Count, 5);

				object[,] a = new object[list.Count, 5];

				for (int i = 0; i < list.Count; i++)
				{
					object[] v = (object[])list[i];

					for (int j = 0; j < 5; j++)
						a[i, j] = v[j];
				}

				oRange.Value2 = a;
			}
			
			// save it.
			if (createNewSheet || list.Count > 0)
			{
				if (xlsExits)
					oWB.Save();
				else
					oWB.SaveAs(XLSFile, Excel.XlFileFormat.xlXMLSpreadsheet, missing, missing, missing, missing, Excel.XlSaveAsAccessMode.xlExclusive, missing, missing, missing, missing, missing);
			}

			oWB.Close(null, null, null);
			oWBs.Close();
			oEA.Quit();
		}

		public DateTime LastModified
		{
			get
			{
				if (File.Exists(_xlsFile))
					return File.GetLastWriteTime(_xlsFile);
				else
					return DateTime.Now;
			}
		}

		private string _xlsFile = string.Empty;

		public string XLSFile
		{
			get
			{
				if (_xlsFile == string.Empty)
				{
					if (IsSupportdCultureInfo(System.Globalization.CultureTypes.AllCultures, _culture))
						_xlsFile = Path.Combine(SampleLRM.ResourceRepositoryPath, _culture + ".xls");
					else
						_xlsFile = Path.Combine(SampleLRM.ResourceRepositoryPath, "Excluded.xls");
				}

				return _xlsFile;
			}
			set
			{
				_xlsFile = value;
			}
		}

		private static Hashtable _allCultures = null;
		private static Hashtable _specificCultures = null;

		//  Find out if a culture is any supported culture or if it is a specific culture.
		internal static bool IsSupportdCultureInfo(System.Globalization.CultureTypes type, string culture)
		{
			// cache two kinds of supported cultures, any type or specific.
			if (_allCultures == null)
			{
				_allCultures = new Hashtable();
				_specificCultures = new Hashtable();

				foreach (CultureInfo c in CultureInfo.GetCultures(CultureTypes.AllCultures))
				{
					_allCultures.Add(c.Name, 1);
					if (!c.IsNeutralCulture)
						_specificCultures.Add(c.Name, 1);
				}
			}

			if (type == CultureTypes.AllCultures)
				return _allCultures.ContainsKey(culture);		
			else if (type == CultureTypes.SpecificCultures)
				return _specificCultures.ContainsKey(culture);
			else
				throw new ArgumentException("CultureType not supported.");
		}
	}
}
